// SPDX-License-Identifier: GPL-3.0-or-later
// Copyright (c) 2025 Abel Luck <abel@guardianproject.info>

package provider

import (
	"context"
	"fmt"

	"github.com/hashicorp/terraform-plugin-framework/datasource"
	"github.com/hashicorp/terraform-plugin-framework/datasource/schema"
	"github.com/hashicorp/terraform-plugin-framework/types"
	"github.com/hashicorp/terraform-plugin-log/tflog"
)

// Ensure provider defined types fully satisfy framework interfaces.
var _ datasource.DataSource = &TorObfs4BridgeLineDataSource{}

func NewTorObfs4BridgeLineDataSource() datasource.DataSource {
	return &TorObfs4BridgeLineDataSource{}
}

// TorObfs4BridgeLineDataSource defines the data source implementation.
type TorObfs4BridgeLineDataSource struct{}

// TorObfs4BridgeLineDataSourceModel describes the data source data model.
type TorObfs4BridgeLineDataSourceModel struct {
	Id                      types.String `tfsdk:"id"`
	IpAddress               types.String `tfsdk:"ip_address"`
	Port                    types.Int64  `tfsdk:"port"`
	IdentityFingerprintSha1 types.String `tfsdk:"identity_fingerprint_sha1"`
	Obfs4StateCertificate   types.String `tfsdk:"obfs4_state_certificate"`
	Obfs4StateIatMode       types.Int64  `tfsdk:"obfs4_state_iat_mode"`
	BridgeLine              types.String `tfsdk:"bridge_line"`
}

func (d *TorObfs4BridgeLineDataSource) Metadata(ctx context.Context, req datasource.MetadataRequest, resp *datasource.MetadataResponse) {
	resp.TypeName = req.ProviderTypeName + "_obfs4_bridge_line"
}

func (d *TorObfs4BridgeLineDataSource) Schema(ctx context.Context, req datasource.SchemaRequest, resp *datasource.SchemaResponse) {
	resp.Schema = schema.Schema{
		MarkdownDescription: "Generates a complete Tor bridge line using obfs4 state and network details",

		Attributes: map[string]schema.Attribute{
			"id": schema.StringAttribute{
				Computed:            true,
				MarkdownDescription: "Data source identifier",
			},
			"ip_address": schema.StringAttribute{
				Required:            true,
				MarkdownDescription: "Bridge IP address",
			},
			"port": schema.Int64Attribute{
				Required:            true,
				MarkdownDescription: "Bridge port number",
			},
			"identity_fingerprint_sha1": schema.StringAttribute{
				Required:            true,
				MarkdownDescription: "SHA1 fingerprint of the RSA identity key",
			},
			"obfs4_state_certificate": schema.StringAttribute{
				Required:            true,
				MarkdownDescription: "Base64-encoded certificate from tor_obfs4_state resource",
			},
			"obfs4_state_iat_mode": schema.Int64Attribute{
				Required:            true,
				MarkdownDescription: "Inter-Arrival Time mode from tor_obfs4_state resource",
			},
			"bridge_line": schema.StringAttribute{
				Computed:            true,
				MarkdownDescription: "Complete bridge line ready for client use",
			},
		},
	}
}

func (d *TorObfs4BridgeLineDataSource) Configure(ctx context.Context, req datasource.ConfigureRequest, resp *datasource.ConfigureResponse) {
	// No configuration needed for this data source
}

func (d *TorObfs4BridgeLineDataSource) Read(ctx context.Context, req datasource.ReadRequest, resp *datasource.ReadResponse) {
	var data TorObfs4BridgeLineDataSourceModel

	// Read Terraform configuration data into the model
	resp.Diagnostics.Append(req.Config.Get(ctx, &data)...)
	if resp.Diagnostics.HasError() {
		return
	}

	// Generate bridge line using the provided inputs
	bridgeLine := fmt.Sprintf("Bridge obfs4 %s:%d %s cert=%s iat-mode=%d",
		data.IpAddress.ValueString(),
		data.Port.ValueInt64(),
		data.IdentityFingerprintSha1.ValueString(),
		data.Obfs4StateCertificate.ValueString(),
		data.Obfs4StateIatMode.ValueInt64())

	// Set computed values
	data.Id = types.StringValue(fmt.Sprintf("bridge-%s-%d", data.IpAddress.ValueString(), data.Port.ValueInt64()))
	data.BridgeLine = types.StringValue(bridgeLine)

	tflog.Trace(ctx, "read bridge line data source")

	// Save data into Terraform state
	resp.Diagnostics.Append(resp.State.Set(ctx, &data)...)
}