// SPDX-License-Identifier: GPL-3.0-or-later
// Copyright (c) 2025 Abel Luck <abel@guardianproject.info>

package provider

import (
	"fmt"
	"regexp"
	"testing"

	"github.com/hashicorp/terraform-plugin-testing/helper/resource"
	"github.com/hashicorp/terraform-plugin-testing/terraform"
)

func TestTorObfs4BridgeLineDataSource(t *testing.T) {
	resource.Test(t, resource.TestCase{
		PreCheck:                 func() { testAccPreCheck(t) },
		ProtoV6ProviderFactories: testAccProtoV6ProviderFactories,
		Steps: []resource.TestStep{
			// Read testing
			{
				Config: testTorObfs4BridgeLineDataSourceConfig(),
				Check: resource.ComposeAggregateTestCheckFunc(
					// Check that the data source was created
					resource.TestCheckResourceAttrSet("data.tor_obfs4_bridge_line.test", "id"),
					
					// Check input values are preserved
					resource.TestCheckResourceAttr("data.tor_obfs4_bridge_line.test", "ip_address", "192.0.2.1"),
					resource.TestCheckResourceAttr("data.tor_obfs4_bridge_line.test", "port", "443"),
					
					// Check computed values are generated
					resource.TestCheckResourceAttrSet("data.tor_obfs4_bridge_line.test", "bridge_line"),
					
					// Check bridge line format
					resource.TestMatchResourceAttr("data.tor_obfs4_bridge_line.test", "bridge_line", 
						regexp.MustCompile(`^Bridge obfs4 192\.0\.2\.1:443 [0-9a-f]{40} cert=[A-Za-z0-9+/]+ iat-mode=[0-2]$`)),
					
					// Check that input values are used correctly
					resource.TestCheckResourceAttrPair("data.tor_obfs4_bridge_line.test", "identity_fingerprint_sha1", "tor_relay_identity_rsa.test", "public_key_fingerprint_sha1"),
					resource.TestCheckResourceAttrPair("data.tor_obfs4_bridge_line.test", "obfs4_state_certificate", "tor_obfs4_state.test", "certificate"),
					resource.TestCheckResourceAttrPair("data.tor_obfs4_bridge_line.test", "obfs4_state_iat_mode", "tor_obfs4_state.test", "iat_mode"),
					
					// Validate bridge line consistency
					testTorObfs4BridgeLineConsistency("data.tor_obfs4_bridge_line.test"),
				),
			},
		},
	})
}

func TestTorObfs4BridgeLineDataSourceWithCustomIATMode(t *testing.T) {
	resource.Test(t, resource.TestCase{
		PreCheck:                 func() { testAccPreCheck(t) },
		ProtoV6ProviderFactories: testAccProtoV6ProviderFactories,
		Steps: []resource.TestStep{
			{
				Config: testTorObfs4BridgeLineDataSourceConfigWithIATMode(2),
				Check: resource.ComposeAggregateTestCheckFunc(
					resource.TestCheckResourceAttr("data.tor_obfs4_bridge_line.test", "obfs4_state_iat_mode", "2"),
					resource.TestMatchResourceAttr("data.tor_obfs4_bridge_line.test", "bridge_line", 
						regexp.MustCompile(`iat-mode=2$`)),
					testTorObfs4BridgeLineConsistency("data.tor_obfs4_bridge_line.test"),
				),
			},
		},
	})
}

func TestTorObfs4BridgeLineDataSourceMultiplePorts(t *testing.T) {
	resource.Test(t, resource.TestCase{
		PreCheck:                 func() { testAccPreCheck(t) },
		ProtoV6ProviderFactories: testAccProtoV6ProviderFactories,
		Steps: []resource.TestStep{
			{
				Config: testTorObfs4BridgeLineDataSourceConfigMultiplePorts(),
				Check: resource.ComposeAggregateTestCheckFunc(
					// Check first bridge line
					resource.TestCheckResourceAttr("data.tor_obfs4_bridge_line.test1", "ip_address", "192.0.2.1"),
					resource.TestCheckResourceAttr("data.tor_obfs4_bridge_line.test1", "port", "443"),
					resource.TestMatchResourceAttr("data.tor_obfs4_bridge_line.test1", "bridge_line", 
						regexp.MustCompile(`^Bridge obfs4 192\.0\.2\.1:443`)),
					
					// Check second bridge line (same obfs4_state, different port)
					resource.TestCheckResourceAttr("data.tor_obfs4_bridge_line.test2", "ip_address", "192.0.2.1"),
					resource.TestCheckResourceAttr("data.tor_obfs4_bridge_line.test2", "port", "80"),
					resource.TestMatchResourceAttr("data.tor_obfs4_bridge_line.test2", "bridge_line", 
						regexp.MustCompile(`^Bridge obfs4 192\.0\.2\.1:80`)),
					
					// Verify both use the same certificate and fingerprint (same obfs4_state)
					resource.TestCheckResourceAttrPair("data.tor_obfs4_bridge_line.test1", "obfs4_state_certificate", "data.tor_obfs4_bridge_line.test2", "obfs4_state_certificate"),
					resource.TestCheckResourceAttrPair("data.tor_obfs4_bridge_line.test1", "identity_fingerprint_sha1", "data.tor_obfs4_bridge_line.test2", "identity_fingerprint_sha1"),
				),
			},
		},
	})
}

// Test configuration functions
func testTorObfs4BridgeLineDataSourceConfig() string {
	return `
resource "tor_relay_identity_rsa" "test" {}

resource "tor_relay_identity_ed25519" "test" {}

resource "tor_obfs4_state" "test" {
  rsa_identity_private_key     = tor_relay_identity_rsa.test.private_key_pem
  ed25519_identity_private_key = tor_relay_identity_ed25519.test.private_key_pem
}

data "tor_obfs4_bridge_line" "test" {
  ip_address                  = "192.0.2.1"
  port                        = 443
  identity_fingerprint_sha1   = tor_relay_identity_rsa.test.public_key_fingerprint_sha1
  obfs4_state_certificate     = tor_obfs4_state.test.certificate
  obfs4_state_iat_mode        = tor_obfs4_state.test.iat_mode
}
`
}

func testTorObfs4BridgeLineDataSourceConfigWithIATMode(iatMode int) string {
	return fmt.Sprintf(`
resource "tor_relay_identity_rsa" "test" {}

resource "tor_relay_identity_ed25519" "test" {}

resource "tor_obfs4_state" "test" {
  rsa_identity_private_key     = tor_relay_identity_rsa.test.private_key_pem
  ed25519_identity_private_key = tor_relay_identity_ed25519.test.private_key_pem
  iat_mode = %d
}

data "tor_obfs4_bridge_line" "test" {
  ip_address                  = "192.0.2.1"
  port                        = 443
  identity_fingerprint_sha1   = tor_relay_identity_rsa.test.public_key_fingerprint_sha1
  obfs4_state_certificate     = tor_obfs4_state.test.certificate
  obfs4_state_iat_mode        = tor_obfs4_state.test.iat_mode
}
`, iatMode)
}

func testTorObfs4BridgeLineDataSourceConfigMultiplePorts() string {
	return `
resource "tor_relay_identity_rsa" "test" {}

resource "tor_relay_identity_ed25519" "test" {}

resource "tor_obfs4_state" "test" {
  rsa_identity_private_key     = tor_relay_identity_rsa.test.private_key_pem
  ed25519_identity_private_key = tor_relay_identity_ed25519.test.private_key_pem
}

data "tor_obfs4_bridge_line" "test1" {
  ip_address                  = "192.0.2.1"
  port                        = 443
  identity_fingerprint_sha1   = tor_relay_identity_rsa.test.public_key_fingerprint_sha1
  obfs4_state_certificate     = tor_obfs4_state.test.certificate
  obfs4_state_iat_mode        = tor_obfs4_state.test.iat_mode
}

data "tor_obfs4_bridge_line" "test2" {
  ip_address                  = "192.0.2.1"
  port                        = 80
  identity_fingerprint_sha1   = tor_relay_identity_rsa.test.public_key_fingerprint_sha1
  obfs4_state_certificate     = tor_obfs4_state.test.certificate
  obfs4_state_iat_mode        = tor_obfs4_state.test.iat_mode
}
`
}

// Custom test check functions
func testTorObfs4BridgeLineConsistency(resourceName string) resource.TestCheckFunc {
	return func(s *terraform.State) error {
		rs, ok := s.RootModule().Resources[resourceName]
		if !ok {
			return fmt.Errorf("Resource not found: %s", resourceName)
		}
		
		ipAddress := rs.Primary.Attributes["ip_address"]
		port := rs.Primary.Attributes["port"]
		fingerprint := rs.Primary.Attributes["identity_fingerprint_sha1"]
		certificate := rs.Primary.Attributes["obfs4_state_certificate"]
		iatMode := rs.Primary.Attributes["obfs4_state_iat_mode"]
		bridgeLine := rs.Primary.Attributes["bridge_line"]
		
		// Verify bridge line format matches expected pattern
		expectedBridgeLine := fmt.Sprintf("Bridge obfs4 %s:%s %s cert=%s iat-mode=%s", 
			ipAddress, port, fingerprint, certificate, iatMode)
		
		if bridgeLine != expectedBridgeLine {
			return fmt.Errorf("Bridge line doesn't match expected format.\nExpected: %s\nActual: %s", 
				expectedBridgeLine, bridgeLine)
		}
		
		return nil
	}
}